@@ -42,7 +42,8 @@ import numpy as np
 from phonopy.structure.atoms import PhonopyAtoms as Atoms
 from phonopy.structure.atoms import symbol_map, atom_data
 from phonopy.structure.cells import get_primitive, get_supercell
-from phonopy.structure.symmetry import Symmetry
+from phonopy.structure.symmetry import (Symmetry, get_site_symmetry,
+                                        get_pointgroup_operations)
 from phonopy.harmonic.force_constants import similarity_transformation
 from phonopy.file_IO import (write_FORCE_SETS, write_force_constants_to_hdf5,
                              write_FORCE_CONSTANTS)
@@ -66,7 +67,7 @@ def parse_set_of_forces(num_atoms,
             if verbose:
                 sys.stdout.write("%d " % (count + 1))
             count += 1
-    
+
             if not check_forces(force_sets[-1], num_atoms, filename):
                 is_parsed = False
 
@@ -104,16 +105,16 @@ def get_drift_forces(forces, filename=None, verbose=True):
 
     return drift_force
 
-def create_FORCE_CONSTANTS(filename, options, log_level):
-    vasprun = Vasprun(io.open(filename, "rb"))
-    fc_and_atom_types = vasprun.read_force_constants()
+def create_FORCE_CONSTANTS(filename, is_hdf5, log_level):
+    fc_and_atom_types = parse_force_constants(filename)
+
     if not fc_and_atom_types:
         print('')
         print("\'%s\' dones not contain necessary information." % filename)
         return 1
 
     force_constants, atom_types = fc_and_atom_types
-    if options.is_hdf5:
+    if is_hdf5:
         try:
             import h5py
         except ImportError:
@@ -133,6 +134,21 @@ def create_FORCE_CONSTANTS(filename, options, log_level):
         print("Atom types: %s" % (" ".join(atom_types)))
     return 0
 
+def parse_force_constants(filename):
+    """Return force constants and chemical elements
+
+    Args:
+        filename (str): Filename
+
+    Returns:
+        tuple: force constants and chemical elements
+
+    """
+
+    vasprun = Vasprun(io.open(filename, "rb"))
+    return vasprun.read_force_constants()
+
+
 #
 # read VASP POSCAR
 #
@@ -224,7 +240,7 @@ def _expand_symbols(num_atoms, symbols=None):
 # write vasp POSCAR
 #
 def write_vasp(filename, atoms, direct=True):
-    lines = _get_vasp_structure(atoms, direct=direct)
+    lines = get_vasp_structure_lines(atoms, direct=direct)
     with open(filename, 'w') as w:
         w.write("\n".join(lines))
 
@@ -287,17 +303,22 @@ def sort_positions_by_symbols(symbols, positions):
                 num_atoms[i] += 1
     return num_atoms, reduced_symbols, np.array(sorted_positions), sort_list
 
-def _get_vasp_structure(atoms, direct=True):
+def get_vasp_structure_lines(atoms, direct=True, is_vasp5=False):
     (num_atoms,
      symbols,
      scaled_positions,
      sort_list) = sort_positions_by_symbols(atoms.get_chemical_symbols(),
                                             atoms.get_scaled_positions())
     lines = []
-    lines.append(" ".join(["%s" % s for s in symbols]))
+    if is_vasp5:
+        lines.append("generated by phonopy")
+    else:
+        lines.append(" ".join(["%s" % s for s in symbols]))
     lines.append("   1.0")
     for a in atoms.get_cell():
         lines.append("  %21.16f %21.16f %21.16f" % tuple(a))
+    if is_vasp5:
+        lines.append(" ".join(["%s" % s for s in symbols]))
     lines.append(" ".join(["%4d" % n for n in num_atoms]))
     lines.append("Direct")
     lines += _get_scaled_positions_lines(scaled_positions)
@@ -356,16 +377,26 @@ def get_born_OUTCAR(poscar_filename="POSCAR",
         borns, epsilon = _read_born_and_epsilon(filename)
 
     num_atom = len(borns)
-    assert num_atom == ucell.get_number_of_atoms()
+    assert num_atom == ucell.get_number_of_atoms(), \
+        "num_atom %d != len(borns) %d" % (ucell.get_number_of_atoms(),
+                                          len(borns))
 
     if symmetrize_tensors:
-        lattice = ucell.get_cell().T
+        lattice = ucell.get_cell()
         positions = ucell.get_scaled_positions()
         u_sym = Symmetry(ucell, is_symmetry=is_symmetry, symprec=symprec)
-        point_sym = [similarity_transformation(lattice, r)
-                     for r in u_sym.get_pointgroup_operations()]
-        epsilon = _symmetrize_tensor(epsilon, point_sym)
-        borns = _symmetrize_borns(borns, u_sym, lattice, positions, symprec)
+        rotations = u_sym.get_symmetry_operations()['rotations']
+        translations = u_sym.get_symmetry_operations()['translations']
+        ptg_ops = get_pointgroup_operations(rotations)
+        epsilon = symmetrize_2nd_rank_tensor(epsilon,
+                                             ptg_ops,
+                                             lattice)
+        borns = symmetrize_borns(borns,
+                                 rotations,
+                                 translations,
+                                 lattice,
+                                 positions,
+                                 symprec)
 
     inv_smat = np.linalg.inv(smat)
     scell = get_supercell(ucell, smat, symprec=symprec)
@@ -383,22 +414,22 @@ def _read_born_and_epsilon(filename):
     with open(filename) as outcar:
         borns = []
         epsilon = []
-    
+
         while True:
             line = outcar.readline()
             if not line:
                 break
-    
+
             if "NIONS" in line:
                 num_atom = int(line.split()[11])
-    
+
             if "MACROSCOPIC STATIC DIELECTRIC TENSOR" in line:
                 epsilon = []
                 outcar.readline()
                 epsilon.append([float(x) for x in outcar.readline().split()])
                 epsilon.append([float(x) for x in outcar.readline().split()])
                 epsilon.append([float(x) for x in outcar.readline().split()])
-    
+
             if "BORN" in line:
                 outcar.readline()
                 line = outcar.readline()
@@ -413,22 +444,28 @@ def _read_born_and_epsilon(filename):
                                      for x in outcar.readline().split()][1:])
                         outcar.readline()
                         borns.append(born)
-    
+
         borns = np.array(borns, dtype='double')
         epsilon = np.array(epsilon, dtype='double')
-    
+
     return borns, epsilon
 
-def _symmetrize_borns(borns, u_sym, lattice, positions, symprec):
+def symmetrize_borns(borns,
+                     rotations,
+                     translations,
+                     lattice,
+                     positions,
+                     symprec):
     borns_orig = borns.copy()
     for i, Z in enumerate(borns):
-        site_sym = [similarity_transformation(lattice, r)
-                    for r in u_sym.get_site_symmetry(i)]
-        Z = _symmetrize_tensor(Z, site_sym)
+        site_sym = get_site_symmetry(i,
+                                     lattice,
+                                     positions,
+                                     rotations,
+                                     translations,
+                                     symprec)
+        Z = symmetrize_2nd_rank_tensor(Z, site_sym, lattice)
 
-    rotations = u_sym.get_symmetry_operations()['rotations']
-    translations = u_sym.get_symmetry_operations()['translations']
-    map_atoms = u_sym.get_map_atoms()
     borns_copy = np.zeros_like(borns)
     for i in range(len(borns)):
         count = 0
@@ -436,8 +473,9 @@ def _symmetrize_borns(borns, u_sym, lattice, positions, symprec):
             count += 1
             diff = np.dot(positions, r.T) + t - positions[i]
             diff -= np.rint(diff)
-            j = np.nonzero((np.abs(diff) < symprec).all(axis=1))[0][0]
-            r_cart = similarity_transformation(lattice, r)
+            dist = np.sqrt(np.sum(np.dot(diff, lattice) ** 2, axis=1))
+            j = np.nonzero(dist < symprec)[0][0]
+            r_cart = similarity_transformation(lattice.T, r)
             borns_copy[i] += similarity_transformation(r_cart, borns[j])
         borns_copy[i] /= count
 
@@ -449,11 +487,21 @@ def _symmetrize_borns(borns, u_sym, lattice, positions, symprec):
         sys.stderr.write(
             "Born effective charge symmetrization might go wrong.\n")
 
+        sys.stderr.write("Sum of Born charges:\n")
+        sys.stderr.write(str(borns_orig.sum(axis=0)))
+        sys.stderr.write("\n")
+
+        # for b_o, b in zip(borns_orig, borns):
+        #     sys.stderr.write(str(b - b_o))
+        #     sys.stderr.write("\n")
+
     return borns
 
-def _symmetrize_tensor(tensor, symmetry_operations):
+def symmetrize_2nd_rank_tensor(tensor, symmetry_operations, lattice):
+    sym_cart = [similarity_transformation(lattice.T, r)
+                for r in symmetry_operations]
     sum_tensor = np.zeros_like(tensor)
-    for sym in symmetry_operations:
+    for sym in sym_cart:
         sum_tensor += similarity_transformation(sym, tensor)
     return sum_tensor / len(symmetry_operations)
 
@@ -490,7 +538,7 @@ class Vasprun(object):
     def read_force_constants(self):
         vasprun = self._parse_etree_vasprun_xml()
         return self._get_force_constants(vasprun)
-    
+
     def _get_forces(self, vasprun_etree):
         """
         vasprun_etree = etree.iterparse(fileptr, tag='varray')
@@ -501,7 +549,7 @@ class Vasprun(object):
                 for v in element:
                     forces.append([float(x) for x in v.text.split()])
         return np.array(forces)
-    
+
     def _get_force_constants(self, vasprun_etree):
         fc_tmp = None
         num_atom = 0
@@ -514,14 +562,14 @@ class Vasprun(object):
                     masses = []
                     for n, m in zip(num_atoms, elem_masses):
                         masses += [m] * n
-    
+
             # Get Hessian matrix (normalized by masses)
             if element.tag == 'varray':
                 if element.attrib['name'] == 'hessian':
                     fc_tmp = []
                     for v in element.findall('./v'):
                         fc_tmp.append([float(x) for x in v.text.strip().split()])
-    
+
         if fc_tmp is None:
             return False
         else:
@@ -530,24 +578,24 @@ class Vasprun(object):
                 return False
             # num_atom = fc_tmp.shape[0] / 3
             force_constants = np.zeros((num_atom, num_atom, 3, 3), dtype='double')
-    
+
             for i in range(num_atom):
                 for j in range(num_atom):
                     force_constants[i, j] = fc_tmp[i*3:(i+1)*3, j*3:(j+1)*3]
-    
+
             # Inverse normalization by atomic weights
             for i in range(num_atom):
                 for j in range(num_atom):
                     force_constants[i, j] *= -np.sqrt(masses[i] * masses[j])
-    
+
             return force_constants, elements
-    
+
     def _get_atomtypes(self, element):
         atom_types = []
         masses = []
         valences = []
         num_atoms = []
-    
+
         if element.tag == 'array':
             if 'name' in element.attrib:
                 if element.attrib['name'] == 'atomtypes':
@@ -558,7 +606,7 @@ class Vasprun(object):
                         masses.append(float(atom_info[2]))
                         valences.append(float(atom_info[3]))
                     return num_atoms, atom_types, masses, valences
-    
+
         return None
 
     def _parse_etree_vasprun_xml(self, tag=None):
@@ -566,7 +614,7 @@ class Vasprun(object):
             return self._parse_by_etree(VasprunWrapper(self._fileptr), tag=tag)
         else:
             return self._parse_by_etree(self._fileptr, tag=tag)
-    
+
     def _parse_by_etree(self, fileptr, tag=None):
         try:
             import xml.etree.cElementTree as etree
@@ -578,13 +626,13 @@ class Vasprun(object):
             print("For creating FORCE_SETS file with Python 2.4, you can use "
                   "phonopy 1.8.5.1 with python-lxml .")
             sys.exit(1)
-    
+
     def _parse_expat_vasprun_xml(self):
         if self._is_version528():
             return self._parse_by_expat(VasprunWrapper(self._fileptr))
         else:
             return self._parse_by_expat(self._fileptr)
-    
+
     def _parse_by_expat(self, fileptr):
         vasprun = VasprunxmlExpat(fileptr)
         vasprun.parse()
@@ -616,6 +664,8 @@ class VasprunxmlExpat(object):
         self._is_symbols = False
         self._is_basis = False
         self._is_energy = False
+        self._is_k_weights = False
+        self._is_eigenvalues = False
         self._is_epsilon = False
         self._is_born = False
 
@@ -624,9 +674,12 @@ class VasprunxmlExpat(object):
         self._is_rc = False
         self._is_c = False
         self._is_set = False
+        self._is_r = False
 
         self._is_scstep = False
         self._is_structure = False
+        self._is_projected = False
+        self._is_proj_eig = False
 
         self._all_forces = []
         self._all_stress = []
@@ -642,6 +695,11 @@ class VasprunxmlExpat(object):
         self._energies = None
         self._epsilon = None
         self._born_atom = None
+        self._k_weights = None
+        self._eigenvalues = None
+        self._eig_state = [0, 0]
+        self._projectors = None
+        self._proj_state = [0, 0, 0]
 
         self._p = xml.parsers.expat.ParserCreate()
         self._p.buffer_text = True
@@ -690,6 +748,15 @@ class VasprunxmlExpat(object):
     def get_energies(self):
         return np.array(self._all_energies)
 
+    def get_k_weights(self):
+        return self._k_weights
+
+    def get_eigenvalues(self):
+        return self._eigenvalues
+
+    def get_projectors(self):
+        return self._projectors
+
     def _start_element(self, name, attrs):
         # Used not to collect energies in <scstep>
         if name == 'scstep':
@@ -707,7 +774,8 @@ class VasprunxmlExpat(object):
             self._is_epsilon or
             self._is_born or
             self._is_positions or
-            self._is_basis):
+            self._is_basis,
+            self._is_k_weights):
             if name == 'v':
                 self._is_v = True
 
@@ -721,6 +789,10 @@ class VasprunxmlExpat(object):
                     self._is_stress = True
                     self._stress = []
 
+                if attrs['name'] == 'weights':
+                    self._is_k_weights = True
+                    self._k_weights = []
+
                 if attrs['name'] == 'epsilon':
                     self._is_epsilon = True
                     self._epsilon = []
@@ -759,6 +831,50 @@ class VasprunxmlExpat(object):
                 if attrs['name'] == 'born_charges':
                     self._is_born = True
 
+        if self._is_projected and not self._is_proj_eig:
+            if name == 'set':
+                if 'comment' in attrs.keys():
+                    if 'spin' in attrs['comment']:
+                        self._projectors.append([])
+                        spin_num = int(attrs['comment'].replace("spin", ''))
+                        self._proj_state = [spin_num - 1, -1, -1]
+                    if 'kpoint' in attrs['comment']:
+                        self._projectors[self._proj_state[0]].append([])
+                        k_num = int(attrs['comment'].split()[1])
+                        self._proj_state[1:3] = k_num - 1, -1
+                    if 'band' in attrs['comment']:
+                        s, k = self._proj_state[:2]
+                        self._projectors[s][k].append([])
+                        b_num = int(attrs['comment'].split()[1])
+                        self._proj_state[2] = b_num - 1
+            if name == 'r':
+                self._is_r = True
+
+        if self._is_eigenvalues:
+            if name == 'set':
+                if 'comment' in attrs.keys():
+                    if 'spin' in attrs['comment']:
+                        self._eigenvalues.append([])
+                        spin_num = int(attrs['comment'].split()[1])
+                        self._eig_state = [spin_num - 1, -1]
+                    if 'kpoint' in attrs['comment']:
+                        self._eigenvalues[self._eig_state[0]].append([])
+                        k_num = int(attrs['comment'].split()[1])
+                        self._eig_state[1] = k_num - 1
+            if name == 'r':
+                self._is_r = True
+
+        if name == 'projected':
+            self._is_projected = True
+            self._projectors = []
+
+        if name == 'eigenvalues':
+            if self._is_projected:
+                self._is_proj_eig = True
+            else:
+                self._is_eigenvalues = True
+                self._eigenvalues = []
+
     def _end_element(self, name):
         if name == 'scstep':
             self._is_scstep = False
@@ -775,6 +891,9 @@ class VasprunxmlExpat(object):
                 self._is_stress = False
                 self._all_stress.append(self._stress)
 
+            if self._is_k_weights:
+                self._is_k_weights = False
+
             if self._is_positions:
                 self._is_positions = False
                 self._all_points.append(np.transpose(self._points))
@@ -811,6 +930,18 @@ class VasprunxmlExpat(object):
         if name == 'c':
             self._is_c = False
 
+        if name == 'r':
+            self._is_r = False
+
+        if name == 'projected':
+            self._is_projected = False
+
+        if name == 'eigenvalues':
+            if self._is_projected:
+                self._is_proj_eig = False
+            else:
+                self._is_eigenvalues = False
+
         if name == 'set':
             self._is_set = False
             if self._is_born:
@@ -839,6 +970,9 @@ class VasprunxmlExpat(object):
                 self._lattice.append(
                     [float(x) for x in data.split()])
 
+            if self._is_k_weights:
+                self._k_weights.append(float(data))
+
             if self._is_born:
                 self._born_atom.append(
                     [float(x) for x in data.split()])
@@ -851,6 +985,17 @@ class VasprunxmlExpat(object):
             if self._is_symbols:
                 self._symbols.append(str(data.strip()))
 
+        if self._is_r:
+            if self._is_projected and not self._is_proj_eig:
+                s, k, b = self._proj_state
+                vals = [float(x) for x in data.split()]
+                self._projectors[s][k][b].append(vals)
+            elif self._is_eigenvalues:
+                s, k = self._eig_state
+                vals = [float(x) for x in data.split()]
+                self._eigenvalues[s][k].append(vals)
+
+
 #
 # XDATCAR
 #
